@@ -10,7 +10,7 @@ use warnings;
 use Carp;
 use Device::Modem;
 
-our $VERSION = '0.3';
+our $VERSION = '0.4';
 our @ISA = ("Device::Modem");
 
 =head1 NAME
@@ -19,7 +19,7 @@ Device::Modem::SMSModem - Perl extension for Device::Modem module
 
 =head1 WARNING
 
-This software has a BETA status. It has been carefully tested with Huawei E173, but since AT commands
+This software has a BETA status. It has been carefully tested with Huawei E173 and Huawei E398, but since AT commands
 may be imlemented differently, 100% compliance with all dongles is not guaranteed. See SUPPORT section for details
 
 
@@ -459,26 +459,32 @@ sub init_sms_storage {
 	my $command="AT+CPMS= \"".$name."\", \"".$name."\", \"".$name."\"".Device::Modem::CR;;
 
         $self->_at_send($command);
-	my $result= $self->answer("CPMS", 1000);
-	my @lines;
 
-	#expect something like 
-	# +CPMS: 8,30,8,30,8,30
-	# OK
-	if (! ($result =~ /OK/)) {
-		carp('Failed to set storage');
-	}
-	if($result =~ /CPMS\:\s*(\d+)\,\s*(\d+)/)
+	my ($result, @lines) = $self->parse_answer(qr/OK|ERROR/, 3000);
+	if ($result ne "OK") 
 	{
-		$self->{"capacity"}= $2;
-		$self->{"sms_in_storage"}= $1;
+		carp('Failed to change storage. Making a query to current storage');
+		
+		# lets' try another way- query current storage
+		$self->{"storage_name"}= undef;
+		# Sometimes modem does not allow to change storage
+		$self->read_sms_count();
+	}
+	else  # successful
+	{
+		if($lines[0] =~ /CPMS\:\s*(\d+)\,\s*(\d+)/)
+		{
+			$self->{"capacity"}= $2;
+			$self->{"sms_in_storage"}= $1;
 
-	}                             
-	else
-	{
-		carp("Unable to parse CPMS output");  
+		}                             
+		else
+		{
+			carp("Unable to parse CPMS output");  
+		}
+		$self->{"storage_name"}= $name;
 	}
-	$self->{"storage_name"}= $name;
+	
 
 	# sets the SMS format to TEXT instead of default PDU
 	my $atcmd = "AT+CMGF=1" . Device::Modem::CR;
@@ -571,27 +577,47 @@ sub read_sms_count {
 	my $command="AT+CPMS?".Device::Modem::CR;
         $self->_at_send($command);
 
-	my $result= $self->answer("CPMS", 1000);
+	my $result= $self->answer("CPMS", 3000);
 
 	if (! ($result =~ /OK/)) {
 		carp('Failed to get storage status');
 		return -1;
 	}
 
-	if ($result =~ /CPMS:\s*\"?$storage_name\"?\,(\d+)/) 
+	if(defined $self->{"storage_name"})
 	{
-		$self->{"sms_in_storage"}= $1;
-		return $1;	
+		if ($result =~ /CPMS:\s*\"?$storage_name\"?\,(\d+)/) 
+		{
+			$self->{"sms_in_storage"}= $1;
+			return $1;	
+		}
+		else
+		{
+			carp("Failed to parse CPMS");
+			return -1;
+		
+		}
 	}
 	else
 	{
-		carp("Failed to parse CPMS");
-		return -1;
+		if ($result =~ /CPMS:\s*\"?([A-Z]+)\"?\,(\d+)/) 
+		{
+			$self->{"sms_in_storage"}= $2;
+			$self->{"storage_name"}= $1;
+			return $2;	
+		}
+		else
+		{
+			carp("Failed to parse CPMS");
+			return -1;
+		
+		}
 		
 	}
  
 }
 
+
 =head2 sms_count
 
 =over 4
@@ -741,7 +767,7 @@ sub _at_send {
 =head1 SUPPORT
 
 Feel free to contact me at dmitriii@gmail.com for questions or suggestions.
-The code has been tested against Huawei E173
+The code has been tested against Huawei E173  and Huawei E398
 If you find that your modem is not compatible because of AT commands mismatch 
 (it may be different in different dongles) please provide modem name and attach AT command output.
 
